import { XHR_Config } from './xhr-config';

/**
 * 默认的请求配置
 */
const DEFAULT_XHR_CFG: XHR_Config = {
    timeout: 3000,
    withCredentials: false
};

const xhr = {
    /**
     * 对象转换为 URL 参数列表，用 & 分隔
     * 
     * @param {Object} param JSON 对象 
     * @returns URL 参数列表
     */
    toParams(param: any): string {
        let result: string = "";

        for (let name in param) {
            if (typeof param[name] != "function")
                result += "&" + name + "=" + encodeURIComponent(param[name]);
        }

        return result.substring(1);
    },

    /**
     * 获取 QueryString 的某个参数
     * 
     * @param val 
     * @returns 
   */
    getQuery(val: string): string {
        const w: number = location.hash.indexOf('?');
        const query: string = location.hash.substring(w + 1);
        let vars: string[] = query.split('&');

        for (let i = 0; i < vars.length; i++) {
            const pair = vars[i].split('=');

            if (pair[0] == val)
                return pair[1];
        }

        return '';
    },

    /**
     * XHR GET 请求
     * 
     * @param url 
     * @param cb 
     * @param params 
     */
    get(url: string, cb: Function, params?: {}, cfg: XHR_Config = DEFAULT_XHR_CFG): void {
        let xhr: XMLHttpRequest = initXhr(cfg);

        if (params != null)
            url += '?' + this.toParams(params);

        xhr.open("GET", url, true);
        xhr.onreadystatechange = function () {
            if (xhr.readyState == 4) {
                if ((xhr.status >= 200 && xhr.status < 300) || xhr.status == 304) {
                    cb && cb(JSON.parse(xhr.responseText));
                } else console.error(xhr.responseText);
            }
        };
        xhr.send();
    },

    /**
     * XHR POST 请求
     * 
     * @param url 
     * @param cb 
     * @param params 
     */
    post(url: string, cb: Function, params: string | {}, cfg: XHR_Config = DEFAULT_XHR_CFG): void {
        let xhr: XMLHttpRequest = initXhr(cfg);
        xhr.open("POST", url, true);
        xhr.onreadystatechange = function () {
            if (xhr.readyState == 4) {
                if ((xhr.status >= 200 && xhr.status < 300) || xhr.status == 304) {
                    cb && cb(JSON.parse(xhr.responseText));
                } else errHandle(xhr);
            }
        };

        // 此方法必须在 open()方法和 send() 之间调用
        if (!cfg.contentType) // 如未设置，默认为表单请求
            xhr.setRequestHeader("Content-Type", "application/x-www-form-urlencoded");

        let _params: string;
        if (typeof params != 'string')
            _params = this.toParams(params);
        else
            _params = <string>params;

        xhr.send(_params);
    },

    /**
    * XHR PUT 请求
    * 
    * @param url 
    * @param cb 
    * @param params 
    */
    put(url: string, cb: Function, params: string | {}, cfg: XHR_Config = DEFAULT_XHR_CFG): void {
        let xhr: XMLHttpRequest = initXhr(cfg);
        xhr.open("PUT", url, true);
        xhr.onreadystatechange = function () {
            if (xhr.readyState == 4) {
                if ((xhr.status >= 200 && xhr.status < 300) || xhr.status == 304) {
                    cb && cb(JSON.parse(xhr.responseText));
                } else console.error(xhr.responseText);
            }
        };

        xhr.setRequestHeader("Content-Type", "application/x-www-form-urlencoded");

        let _params: string;
        if (typeof params != 'string')
            _params = this.toParams(params);
        else
            _params = <string>params;

        xhr.send(_params);
    },

    /**
    * XHR GET 请求
    * 
    * @param url 
    * @param cb 
    * @param params 
    */
    dele(url: string, cb: Function, params?: {}, cfg: XHR_Config = DEFAULT_XHR_CFG): void {
        let xhr: XMLHttpRequest = initXhr(cfg);
        if (params != null)
            url += '?' + this.toParams(params);

        xhr.open("DELETE", url, true);
        xhr.onreadystatechange = function (e) {
            if (xhr.readyState == 4) {
                if ((xhr.status >= 200 && xhr.status < 300) || xhr.status == 304) {
                    cb && cb(JSON.parse(xhr.responseText));
                } else console.error(xhr.responseText);
            }
        };

        xhr.send();
    },
};

/**
 * 初始化 XHR
 * 
 * @param cfg 
 * @returns 
 */
function initXhr(cfg: XHR_Config): XMLHttpRequest {
    let xhr: XMLHttpRequest = new XMLHttpRequest();

    if (cfg && cfg.timeout) {
        xhr.timeout = cfg.timeout;
        xhr.ontimeout = (e: ProgressEvent<EventTarget>) => { alert('系统异常，XHR 连接服务超时'); };
    }

    if (cfg && cfg.withCredentials)
        xhr.withCredentials = true;

    return xhr;
}

/**
 * 
 * @param xhr 
 */
 export function errHandle(xhr: XMLHttpRequest): void {
    let msg: string;

    if (xhr.status <= 400) {
        msg = '请求参数错误或者权限不足。';
    } else if (xhr.status <= 500) {
        msg = '服务端异常。';
    } else
        msg = `未知异常，HTTP code:${xhr.status}。`;

    if (!xhr.responseText)
        msg += " 服务端返回空的字符串！";

    console.error(msg, xhr.responseText);
    alert(msg);
}

// export default xhr;